

#-------------------------------------------------------------------------------
#           Modeling proportion data using SPDE-INLA approach 
#           By Idelphonse SODE (PhD Student, Epidemiology),
#             Division of Epidemiology and Biostatistics, SU &
#        Laboratoire de Biomathématiques et Estimations Forestières, UAC
#-------------------------------------------------------------------------------

rm(list = ls(all=TRUE))  

library(INLA)
library(ggplot2)
library(RColorBrewer)
library(dplyr)
library(Hmisc)      

library(SDMTools)
library(geoR)
library(raster)
library(maptools)
data(wrld_simpl)

#-- Importing points and covaraites
df <- read.csv("F:/SpaStat/Cleaned_prop.csv", dec=".")
names(df)

files.env <- list.files("F:/SpaStat/Covariates_All", pattern =".asc", full.names=T)
env.stack <- stack(files.env) 
plot(env.stack)

env.df    <- asc2dataframe(files.env, varnames = names(env.stack))
env.mat   <- as.matrix(env.df)[,-(1:2)]      

#-- Correlation matrix
setwd ("F:/SpaStat/Covariates_All")
all.corr <- rcorr(env.mat, type = "spearman")
write.csv(all.corr$r, file ="F:/SpaStat/Covariates_All/Collinearity/spearman_r.csv")
write.csv(all.corr$P, file ="F:/SpaStat/Covariates_All/Collinearity/spearman_signif.csv")

#-- Covariates at observed locations
covals <- raster::extract(env.stack, df[,2:3])
dataDF <- data.frame(cbind(df, covals))

# Covariates selected from correlation matrix
cov2scale <- c("ben_ppp", "bio1_wc30s", "bio12_wc30s", "bio4_wc30s", "dst_waterway",
              "mimq_wc30s", "srtm_slope", "dst_coastlin") 

data.sc <- dataDF
data.sc[,cov2scale] <- apply(data.sc[,cov2scale], 2, scale)
head(data.sc)

f1 <- as.formula(paste0(" ~ ", paste(cov2scale, collapse = "+")))
f1  

#---------------------------------------
#----- A) Frequentist GLMs ------------
#---------------------------------------

inf.glm0  <- glm(cbind(N_infec, N_tested -N_infec) ~1, family = binomial, data=data.sc)
summary(inf.glm0)

inf.glm1  <- glm(cbind(N_infec, N_tested -N_infec) ~ ben_ppp + bio1_wc30s + bio12_wc30s + bio4_wc30s + dst_waterway + 
                   mimq_wc30s + srtm_slope + dst_coastlin, family = binomial, data=data.sc)
summary(inf.glm1)
#anova(inf.glm0, inf.glm1 , test ="Chisq")

# Keep only covariate with significance at 10% level
summary(step(inf.glm1, direction ="backward"))

#final.cov <- c("ben_ppp", "bio1_wc30s", "bio12_wc30s", "bio4_wc30s", "mimq_wc30s", "dst_coastlin")


#------------------------------------------
#-------- B) Bayesian modeling ------------
#------------------------------------------

#---- a) Bayesian model (without heterogneity components)
form.inf0 <- N_infec ~ 1 + ben_ppp + bio1_wc30s + bio12_wc30s + bio4_wc30s + mimq_wc30s + dst_coastlin
inf.inla0 <- inla(form.inf0, family ="binomial",
                    Ntrials = N_tested, data=data.sc, control.compute =list(dic=TRUE))
summary(inf.inla0)
      
#---- b) Including community-specific random variations
form.inf <- N_infec ~ 1+ ben_ppp + bio1_wc30s + bio12_wc30s + bio4_wc30s + mimq_wc30s+
                   dst_coastlin + f(ID, model="iid")     # dst_waterway + srtm_slope

inf.inla <- inla(form.inf, family ="binomial",
                    Ntrials = N_tested, data=data.sc, control.compute =list(dic=TRUE))
summary(inf.inla)

#- Average local prevalence
prob.inf.avg <- inla.tmarginal(function(x) exp(x)/(1+exp(x)), inf.inla$marginals.fixed[[1]])
inla.zmarginal(prob.inf.avg)           


#--------------------------------------------
##------ c) Bayesian spatial model ----------
#--------------------------------------------

#--- Constructing the mesh
locations <- data.sc[,2:3]
coords    <- as.matrix(locations) 

bnd  <- inla.nonconvex.hull(coords)  
mesh <- inla.mesh.2d(boundary = bnd, offset =c(0.05, 0.10),    # offset=c(0.1,0.4)
                    cutoff = 0.005,  max.edge =c(0.02,0.04))   # max.edge=c(0.1, 0.2)

plot(mesh); points(coords, col="red", cex=1)  
mesh$n

#-- Projector matrix A and spatial effect indexes
A.est <- inla.spde.make.A(mesh = mesh, loc = coords)
dim(A.est)
spde <- inla.spde2.matern(mesh = mesh, alpha = 2)   # alpha=2 for nu=1
s.index    <- inla.spde.make.index(name ="sp.field", n.spde = spde$n.spde)

#spde.pc <- inla.spde2.pcmatern(mesh = mesh,
#                    prior.range = c(0.05, 0.01),    # P(practic.range < 0.05) = 0.01
#                        prior.sigma = c(1, 0.01))   # P(sigma > 1) = 0.01


#--- Stacking all the relevant pieces together ---------
inf.stack.est <- inla.stack(data = list(y = data.sc$N_infec, Ntrials = data.sc$N_tested),   
                              A  = list(A.est, 1, 1, 1, 1, 1, 1, 1),
                              effects = list(c(s.index, list(Intercept =1)),
                                            list(ben_ppp = data.sc$ben_ppp),
                                            list(bio1_wc30s = data.sc$bio1_wc30s),
                                            list(bio12_wc30s = data.sc$bio12_wc30s),
                                            list(bio4_wc30s = data.sc$bio4_wc30s),
                                            list(mimq_wc30s = data.sc$mimq_wc30s),
                                            list(dst_coastlin = data.sc$dst_coastlin),
                                            list(ID = data.sc$ID)),
                             tag ="est")


#---------------------------------
#----- Running SPDE models -------
#---------------------------------

formula  <- y ~ -1 + Intercept + ben_ppp + bio1_wc30s + bio12_wc30s + bio4_wc30s + 
      mimq_wc30s + dst_coastlin +
      f(sp.field, model = spde) + f(ID, model ="iid")

data.inf  <- inla.stack.data(inf.stack.est)

inf.inla.sp <- inla(formula,
                data = data.inf, family ="binomial", Ntrials = data.inf$Ntrials,
                control.family = list(link ="logit"),
                control.predictor = list(A = inla.stack.A(inf.stack.est), compute=TRUE, link=1),
                control.compute   = list(dic=TRUE, cpo=TRUE))

summary(inf.inla.sp)

#-- Variation of prevalence with respect to "dst_coastlin" (in original scale)
exp.coast <- inla.tmarginal(inla.link.invlogit, inf.inla.sp$marginals.fixed[["dst_coastlin"]])
inla.zmarginal(exp.coast)

#-- Mean prevalence at observed locations
index.est <- inla.stack.index(inf.stack.est, tag ="est")$data
fit.loc   <- inf.inla.sp$summary.fitted.values[index.est,"mean"]
fit.loc 


#------------------------------------------------------------------------
#---- Joint estimation/prediction (Blangiardo & Cameletti 2015)
#------------------------------------------------------------------------

# Coordinates ranges for the whole study region
x.lim <- range(coords[,1])
y.lim <- range(coords[,2])
grid.x <- 37    # row number
grid.y <- 36    

pred.grid <- expand.grid(X = seq(x.lim[1], x.lim[2], length.out = grid.x),
                         Y = seq(y.lim[1], y.lim[2], length.out = grid.y))  
pred.grid <- as.matrix(pred.grid)
dim(pred.grid)                  

# Extracting covariates at predictions locations
final.cov    <- c("ben_ppp","bio1_wc30s","bio12_wc30s","bio4_wc30s","mimq_wc30s","dst_coastlin")
pred.covals  <- raster::extract(env.stack, pred.grid[,1:2])

pred.sc    <- apply(pred.covals[,final.cov], 2, scale)
pred.data  <- data.frame(pred.grid, pred.sc, ID = 1:nrow(pred.sc))
head(pred.data)

A.pred <- inla.spde.make.A(mesh = mesh, loc = pred.grid)
dim(A.pred)

##--- Building stack objects
stack.pred.resp <- inla.stack(data = list(y = NA, Ntrials = NA),   
                            A  = list(A.pred, 1, 1, 1, 1, 1, 1, 1),
                            effects = list(c(s.index, list(Intercept =1)),
                                           list(ben_ppp = pred.data$ben_ppp),
                                           list(bio1_wc30s = pred.data$bio1_wc30s),
                                           list(bio12_wc30s = pred.data$bio12_wc30s),
                                           list(bio4_wc30s = pred.data$bio4_wc30s),
                                           list(mimq_wc30s = pred.data$mimq_wc30s),
                                           list(dst_coastlin = pred.data$dst_coastlin),
                                           list(ID = pred.data$ID)),
                            tag ="pred.resp")

# Join together all predictors
join.stack.inf  <- inla.stack(inf.stack.est, stack.pred.resp)
join.data.inf  <- inla.stack.data(join.stack.inf)

##-- Making joint estimation/prediction with INLA
join.inla.inf <- inla(formula, data = inla.stack.data(join.stack.inf), 
              Ntrials = join.data.inf$Ntrials,
              control.family =list(link ="logit"), control.compute = list(config =TRUE), 
              control.results = list(return.marginals.random = FALSE),
              control.predictor = list(A =inla.stack.A(join.stack.inf), compute =TRUE, link=1))

summary(join.inla.inf)


#-------------------------------------------------------------
#--- (a) Summary of the predictive prevalence (mean and SD)
#-------------------------------------------------------------

index.pred.resp   <- inla.stack.index(join.stack.inf, tag ="pred.resp")$data

#-- Posterior mean and SD of etimated proportion
prev.mean.inf  <- join.inla.inf$summary.fitted.values[index.pred.resp, "mean"]
prev.sd.inf    <- join.inla.inf$summary.fitted.values[index.pred.resp, "sd"]

prev.ll.inf  <- join.inla.inf$summary.fitted.values[index.pred.resp, "0.025quant"]
prev.ul.inf  <- join.inla.inf$summary.fitted.values[index.pred.resp, "0.975quant"]
    
# Preview
prev.mean.inf[200:300]
plot(prev.mean.inf, ylab ="Mean prevalence", xlab ="Grid location indexes")

#-- Posterior distribution of the standard deviation of hyparam
post.se.inf <- inla.tmarginal(function(x) sqrt(1/exp(x)), join.inla.inf$internal.marginals.hyperpar[[1]])

# Expected values of post.se and 95% credibility intervals
inla.emarginal(function(x) x, post.se.inf)
inla.zmarginal(post.se.inf)

#-------------------------------------
#--- (b) Mapping of prevalence ---------
#-------------------------------------

imdir  <- "F:/SpaStat/Mapping/"
r      <- raster(extent(env.stack[[1]]), nrow=37, ncol=36)
crs(r) <- crs(wrld_simpl)

#-- Prevalence (mean and SD) ----
r.prev.mean.inf   <- r
r.prev.mean.inf[] <- prev.mean.inf
  
r.prev.sd.inf    <- r
r.prev.sd.inf[]  <- prev.sd.inf

   # Write outputs in Geotiff for other processing in GIS
writeRaster(r.prev.mean.inf, paste0(imdir, "prev_mean_inf.tiff"), type ="GTiff", overwrite =T)
writeRaster(r.prev.sd.inf, paste0(imdir, "prev_sd_inf.tiff"), type ="GTiff", overwrite =T)

   # Plots of results
col.pal  <- colorRampPalette(c("blue", "white", "red"))(10)
plot(r.prev.mean.inf, col= col.pal, main ="Mean prevalence")



